@@ -6,6 +6,10 @@ import subprocess  # nosec
 from abc import ABC, abstractmethod
 from typing import List
 
+from .singularity import get_version as get_singularity_version
+from .singularity import is_version_2_6 as is_singularity_version_2_6
+from .singularity import is_version_3_or_newer as is_singularity_version_3_or_newer
+
 logging.basicConfig(level=logging.INFO)
 _LOGGER = logging.getLogger(__name__)
 
@@ -32,14 +36,12 @@ class ImagePuller(ABC):
             )
         except subprocess.CalledProcessError as err:
             if err.output:
-                raise subprocess.SubprocessError(err.output)
+                raise subprocess.SubprocessError(err.output) from err
             raise err
 
 
 class DockerImagePuller(ImagePuller):
-    """
-    Pull docker image with Docker
-    """
+    """Pull docker image with Docker."""
 
     def get_image_name(self) -> str:
         """Get the name of the tarball."""
@@ -69,9 +71,7 @@ class DockerImagePuller(ImagePuller):
 
 
 class SingularityImagePuller(ImagePuller):
-    """
-    Pull docker image with Singularity
-    """
+    """Pull docker image with Singularity."""
 
     CHARS_TO_REPLACE = ["/"]
     NEW_CHAR = "_"
@@ -79,38 +79,24 @@ class SingularityImagePuller(ImagePuller):
     def __init__(self, req: str, save_directory: str) -> None:
         """Create a Singularity-based software container image downloader."""
         super().__init__(req, save_directory)
-        version = subprocess.check_output(  # nosec
-            ["singularity", "--version"], universal_newlines=True
-        )
-        if version.startswith("singularity version "):
-            version = version[20:]
-        self.version = version
-
-    def _is_version_2_6(self) -> bool:
-        return self.version.startswith("2.6")
-
-    def _is_version_3_or_newer(self) -> bool:
-        return int(self.version[0]) >= 3
 
     def get_image_name(self) -> str:
         """Determine the file name appropriate to the installed version of Singularity."""
         image_name = self.req
         for char in self.CHARS_TO_REPLACE:
             image_name = image_name.replace(char, self.NEW_CHAR)
-        if self._is_version_2_6():
+        if is_singularity_version_2_6():
             suffix = ".img"
-        elif self._is_version_3_or_newer():
+        elif is_singularity_version_3_or_newer():
             suffix = ".sif"
         else:
             raise Exception(
-                "Don't know how to handle this version of singularity: {}.".format(
-                    self.version
-                )
+                f"Don't know how to handle this version of singularity: {get_singularity_version()}."
             )
         return f"{image_name}{suffix}"
 
     def save_docker_image(self) -> None:
-        """Pull down the Docker format software container image and save it in the Singularity image format."""
+        """Pull down the Docker container image in the Singularity image format."""
         _LOGGER.info(f"Pulling {self.req} with Singularity...")
         cmd_pull = [
             "singularity",
