@@ -1,18 +1,62 @@
 # SPDX-License-Identifier: Apache-2.0
 import hashlib
-from typing import IO, Any, List, MutableSequence, Optional, Tuple, Union, cast
+import logging
+from collections import namedtuple
+from typing import IO, Any, Dict, List, MutableSequence, Optional, Tuple, Union, cast
 
 from ruamel import yaml
 from schema_salad.exceptions import ValidationException
-from schema_salad.utils import json_dumps
+from schema_salad.sourceline import SourceLine
+from schema_salad.utils import aslist, json_dumps
 
 import cwl_utils.parser
-import cwl_utils.parser.cwl_v1_2 as cwl
+import cwl_utils.parser.cwl_v1_1 as cwl
 import cwl_utils.parser.utils
 from cwl_utils.errors import WorkflowException
 
 CONTENT_LIMIT: int = 64 * 1024
 
+_logger = logging.getLogger("cwl_utils")
+
+SrcSink = namedtuple("SrcSink", ["src", "sink", "linkMerge", "message"])
+
+
+def _compare_records(
+    src: cwl.RecordSchema, sink: cwl.RecordSchema, strict: bool = False
+) -> bool:
+    """
+    Compare two records, ensuring they have compatible fields.
+
+    This handles normalizing record names, which will be relative to workflow
+    step, so that they can be compared.
+    """
+    srcfields = {cwl.shortname(field.name): field.type for field in (src.fields or {})}
+    sinkfields = {
+        cwl.shortname(field.name): field.type for field in (sink.fields or {})
+    }
+    for key in sinkfields.keys():
+        if (
+            not can_assign_src_to_sink(
+                srcfields.get(key, "null"), sinkfields.get(key, "null"), strict
+            )
+            and sinkfields.get(key) is not None
+        ):
+            _logger.info(
+                "Record comparison failure for %s and %s\n"
+                "Did not match fields for %s: %s and %s",
+                cast(
+                    Union[cwl.InputRecordSchema, cwl.CommandOutputRecordSchema], src
+                ).name,
+                cast(
+                    Union[cwl.InputRecordSchema, cwl.CommandOutputRecordSchema], sink
+                ).name,
+                key,
+                srcfields.get(key),
+                sinkfields.get(key),
+            )
+            return False
+    return True
+
 
 def _compare_type(type1: Any, type2: Any) -> bool:
     if isinstance(type1, cwl.ArraySchema) and isinstance(type2, cwl.ArraySchema):
@@ -26,37 +70,141 @@ def _compare_type(type1: Any, type2: Any) -> bool:
         }
         if fields1.keys() != fields2.keys():
             return False
-        return all((_compare_type(fields1[k], fields2[k]) for k in fields1.keys()))
+        return all(_compare_type(fields1[k], fields2[k]) for k in fields1.keys())
     elif isinstance(type1, MutableSequence) and isinstance(type2, MutableSequence):
         if len(type1) != len(type2):
             return False
         for t1 in type1:
-            if not any((_compare_type(t1, t2) for t2 in type2)):
+            if not any(_compare_type(t1, t2) for t2 in type2):
                 return False
         return True
     else:
         return bool(type1 == type2)
 
 
+def can_assign_src_to_sink(src: Any, sink: Any, strict: bool = False) -> bool:
+    """
+    Check for identical type specifications, ignoring extra keys like inputBinding.
+
+    src: admissible source types
+    sink: admissible sink types
+
+    In non-strict comparison, at least one source type must match one sink type,
+       except for 'null'.
+    In strict comparison, all source types must match at least one sink type.
+    """
+    if src == "Any" or sink == "Any":
+        return True
+    if isinstance(src, cwl.ArraySchema) and isinstance(sink, cwl.ArraySchema):
+        return can_assign_src_to_sink(src.items, sink.items, strict)
+    if isinstance(src, cwl.RecordSchema) and isinstance(sink, cwl.RecordSchema):
+        return _compare_records(src, sink, strict)
+    if isinstance(src, MutableSequence):
+        if strict:
+            for this_src in src:
+                if not can_assign_src_to_sink(this_src, sink):
+                    return False
+            return True
+        for this_src in src:
+            if this_src != "null" and can_assign_src_to_sink(this_src, sink):
+                return True
+        return False
+    if isinstance(sink, MutableSequence):
+        for this_sink in sink:
+            if can_assign_src_to_sink(src, this_sink):
+                return True
+        return False
+    return bool(src == sink)
+
+
+def check_all_types(
+    src_dict: Dict[str, Any],
+    sinks: MutableSequence[Union[cwl.WorkflowStepInput, cwl.WorkflowOutputParameter]],
+    type_dict: Dict[str, Any],
+) -> Dict[str, List[SrcSink]]:
+    """Given a list of sinks, check if their types match with the types of their sources."""
+    validation: Dict[str, List[SrcSink]] = {"warning": [], "exception": []}
+    for sink in sinks:
+        if isinstance(sink, cwl.WorkflowOutputParameter):
+            sourceName = "outputSource"
+            sourceField = sink.outputSource
+        elif isinstance(sink, cwl.WorkflowStepInput):
+            sourceName = "source"
+            sourceField = sink.source
+        else:
+            continue
+        if sourceField is not None:
+            if isinstance(sourceField, MutableSequence):
+                linkMerge = sink.linkMerge or (
+                    "merge_nested" if len(sourceField) > 1 else None
+                )
+                srcs_of_sink = []
+                for parm_id in sourceField:
+                    srcs_of_sink += [src_dict[parm_id]]
+            else:
+                parm_id = cast(str, sourceField)
+                if parm_id not in src_dict:
+                    raise SourceLine(sink, sourceName, ValidationException).makeError(
+                        f"{sourceName} not found: {parm_id}"
+                    )
+                srcs_of_sink = [src_dict[parm_id]]
+                linkMerge = None
+            for src in srcs_of_sink:
+                check_result = check_types(
+                    type_dict[cast(str, src.id)],
+                    type_dict[cast(str, sink.id)],
+                    linkMerge,
+                    getattr(sink, "valueFrom", None),
+                )
+                if check_result == "warning":
+                    validation["warning"].append(SrcSink(src, sink, linkMerge, None))
+                elif check_result == "exception":
+                    validation["exception"].append(SrcSink(src, sink, linkMerge, None))
+    return validation
+
+
+def check_types(
+    srctype: Any,
+    sinktype: Any,
+    linkMerge: Optional[str],
+    valueFrom: Optional[str] = None,
+) -> str:
+    """
+    Check if the source and sink types are correct.
+
+    Acceptable types are "pass", "warning", or "exception".
+    """
+    if valueFrom is not None:
+        return "pass"
+    if linkMerge is None:
+        if can_assign_src_to_sink(srctype, sinktype, strict=True):
+            return "pass"
+        if can_assign_src_to_sink(srctype, sinktype, strict=False):
+            return "warning"
+        return "exception"
+    if linkMerge == "merge_nested":
+        return check_types(
+            cwl.ArraySchema(items=srctype, type="array"), sinktype, None, None
+        )
+    if linkMerge == "merge_flattened":
+        return check_types(merge_flatten_type(srctype), sinktype, None, None)
+    raise ValidationException(f"Invalid value {linkMerge} for linkMerge field.")
+
+
 def content_limit_respected_read_bytes(f: IO[bytes]) -> bytes:
     """
     Read file content up to 64 kB as a byte array.
 
-    Throw exception for larger files (see https://www.commonwl.org/v1.2/Workflow.html#Changelog).
+    Truncate content for larger files.
     """
-    contents = f.read(CONTENT_LIMIT + 1)
-    if len(contents) > CONTENT_LIMIT:
-        raise WorkflowException(
-            "file is too large, loadContents limited to %d bytes" % CONTENT_LIMIT
-        )
-    return contents
+    return f.read(CONTENT_LIMIT)
 
 
 def content_limit_respected_read(f: IO[bytes]) -> str:
     """
     Read file content up to 64 kB as an utf-8 encoded string.
 
-    Throw exception for larger files (see https://www.commonwl.org/v1.2/Workflow.html#Changelog).
+    Truncate content for larger files.
     """
     return content_limit_respected_read_bytes(f).decode("utf-8")
 
@@ -117,12 +265,64 @@ def merge_flatten_type(src: Any) -> Any:
     return cwl.ArraySchema(type="array", items=src)
 
 
+def type_for_step_input(
+    step: cwl.WorkflowStep,
+    in_: cwl.WorkflowStepInput,
+) -> Any:
+    """Determine the type for the given step input."""
+    if in_.valueFrom is not None:
+        return "Any"
+    step_run = cwl_utils.parser.utils.load_step(step)
+    cwl_utils.parser.utils.convert_stdstreams_to_files(step_run)
+    if step_run and step_run.inputs:
+        for step_input in step_run.inputs:
+            if (
+                cast(str, step_input.id).split("#")[-1]
+                == cast(str, in_.id).split("#")[-1]
+            ):
+                input_type = step_input.type
+                if step.scatter is not None and in_.id in aslist(step.scatter):
+                    input_type = cwl.ArraySchema(items=input_type, type="array")
+                return input_type
+    return "Any"
+
+
+def type_for_step_output(
+    step: cwl.WorkflowStep,
+    sourcename: str,
+) -> Any:
+    """Determine the type for the given step output."""
+    step_run = cwl_utils.parser.utils.load_step(step)
+    cwl_utils.parser.utils.convert_stdstreams_to_files(step_run)
+    if step_run and step_run.outputs:
+        for output in step_run.outputs:
+            if (
+                output.id.split("#")[-1].split("/")[-1]
+                == sourcename.split("#")[-1].split("/")[-1]
+            ):
+                output_type = output.type
+                if step.scatter is not None:
+                    if step.scatterMethod == "nested_crossproduct":
+                        for _ in range(len(aslist(step.scatter))):
+                            output_type = cwl.ArraySchema(
+                                items=output_type, type="array"
+                            )
+                    else:
+                        output_type = cwl.ArraySchema(items=output_type, type="array")
+                return output_type
+    raise ValidationException(
+        "param {} not found in {}.".format(
+            sourcename,
+            yaml.main.round_trip_dump(cwl.save(step)),
+        )
+    )
+
+
 def type_for_source(
     process: Union[cwl.CommandLineTool, cwl.Workflow, cwl.ExpressionTool],
     sourcenames: Union[str, List[str]],
     parent: Optional[cwl.Workflow] = None,
     linkMerge: Optional[str] = None,
-    pickValue: Optional[str] = None,
 ) -> Any:
     """Determine the type for the given sourcenames."""
     scatter_context: List[Optional[Tuple[int, str]]] = []
@@ -139,18 +339,12 @@ def type_for_source(
             new_type = cwl.ArraySchema(items=new_type, type="array")
         elif linkMerge == "merge_flattened":
             new_type = merge_flatten_type(new_type)
-        if pickValue is not None:
-            if isinstance(new_type, cwl.ArraySchema):
-                if pickValue in ["first_non_null", "the_only_non_null"]:
-                    new_type = new_type.items
         return new_type
     new_type = []
     for p, sc in zip(params, scatter_context):
-        if isinstance(p, str) and not any((_compare_type(t, p) for t in new_type)):
+        if isinstance(p, str) and not any(_compare_type(t, p) for t in new_type):
             cur_type = p
-        elif hasattr(p, "type") and not any(
-            (_compare_type(t, p.type) for t in new_type)
-        ):
+        elif hasattr(p, "type") and not any(_compare_type(t, p.type) for t in new_type):
             cur_type = p.type
         else:
             cur_type = None
@@ -165,16 +359,13 @@ def type_for_source(
     if len(new_type) == 1:
         new_type = new_type[0]
     if linkMerge == "merge_nested":
-        new_type = cwl.ArraySchema(items=new_type, type="array")
+        return cwl.ArraySchema(items=new_type, type="array")
     elif linkMerge == "merge_flattened":
-        new_type = merge_flatten_type(new_type)
-    elif isinstance(sourcenames, List):
-        new_type = cwl.ArraySchema(items=new_type, type="array")
-    if pickValue is not None:
-        if isinstance(new_type, cwl.ArraySchema):
-            if pickValue in ["first_non_null", "the_only_non_null"]:
-                new_type = new_type.items
-    return new_type
+        return merge_flatten_type(new_type)
+    elif isinstance(sourcenames, List) and len(sourcenames) > 1:
+        return cwl.ArraySchema(items=new_type, type="array")
+    else:
+        return new_type
 
 
 def param_for_source_id(
@@ -210,26 +401,14 @@ def param_for_source_id(
                         == step.id.split("#")[-1]
                         and step.out
                     ):
+                        step_run = cwl_utils.parser.utils.load_step(step)
+                        cwl_utils.parser.utils.convert_stdstreams_to_files(step_run)
                         for outp in step.out:
                             outp_id = outp if isinstance(outp, str) else outp.id
                             if (
                                 outp_id.split("#")[-1].split("/")[-1]
                                 == sourcename.split("#")[-1].split("/")[-1]
                             ):
-                                step_run = step.run
-                                if isinstance(step.run, str):
-                                    step_run = cwl_utils.parser.load_document_by_uri(
-                                        path=target.loadingOptions.fetcher.urljoin(
-                                            base_url=cast(
-                                                str, target.loadingOptions.fileuri
-                                            ),
-                                            url=step.run,
-                                        ),
-                                        loadingOptions=target.loadingOptions,
-                                    )
-                                    cwl_utils.parser.utils.convert_stdstreams_to_files(
-                                        step_run
-                                    )
                                 if step_run and step_run.outputs:
                                     for output in step_run.outputs:
                                         if (
@@ -267,7 +446,7 @@ def param_for_source_id(
         "param {} not found in {}\n{}.".format(
             sourcename,
             yaml.main.round_trip_dump(cwl.save(process)),
-            " or\n {}".format(yaml.main.round_trip_dump(cwl.save(parent)))
+            f" or\n {yaml.main.round_trip_dump(cwl.save(parent))}"
             if parent is not None
             else "",
         )
