@@ -164,7 +164,6 @@ class NodeJSEngine(JSEngine):
         force_docker_pull: bool = False,
         container_engine: str = "docker",
     ) -> Tuple[int, str, str]:
-
         if not hasattr(self.localdata, "procs"):
             self.localdata.procs = {}
 
@@ -230,8 +229,8 @@ class NodeJSEngine(JSEngine):
         stdout_buf = BytesIO()
         stderr_buf = BytesIO()
 
-        rselect = [nodejs.stdout, nodejs.stderr]  # type: List[BytesIO]
-        wselect = [nodejs.stdin]  # type: List[BytesIO]
+        rselect: List[BytesIO] = [nodejs.stdout, nodejs.stderr]
+        wselect: List[BytesIO] = [nodejs.stdin]
 
         def process_finished() -> bool:
             return stdout_buf.getvalue().decode("utf-8").endswith(
@@ -308,7 +307,7 @@ class NodeJSEngine(JSEngine):
 
         if nodejs is None or nodejs is not None and required_node_version is False:
             try:
-                nodeimg = "docker.io/node:slim"
+                nodeimg = "docker.io/node:alpine"
                 if container_engine == "singularity":
                     nodeimg = f"docker://{nodeimg}"
 
@@ -323,10 +322,14 @@ class NodeJSEngine(JSEngine):
                         singularity_cache = os.environ.get("CWL_SINGULARITY_CACHE")
                         if singularity_cache:
                             singularityimgs = glob.glob(
-                                singularity_cache + "/node_slim.sif"
+                                singularity_cache + "/node_alpine.sif"
                             )
                         else:
-                            singularityimgs = glob.glob(os.getcwd() + "/node_slim.sif")
+                            singularityimgs = glob.glob(
+                                os.getcwd() + "/node_alpine.sif"
+                            )
+                        if singularityimgs:
+                            nodeimg = singularityimgs[0]
                     else:
                         raise Exception(
                             f"Unknown container_engine: {container_engine}."
@@ -339,7 +342,7 @@ class NodeJSEngine(JSEngine):
                         len(dockerimgs.split("\n")) <= 1
                     )
                     if need_singularity or need_docker or force_docker_pull:
-                        # pull node:slim docker container
+                        # pull node:alpine docker container
                         nodejs_pull_commands = [container_engine, "pull"]
                         if force_docker_pull:
                             nodejs_pull_commands.append("--force")
@@ -401,15 +404,15 @@ class NodeJSEngine(JSEngine):
                     pass
                 else:
                     raise
-            except subprocess.CalledProcessError:
-                pass
+            except subprocess.CalledProcessError as e:
+                _logger.debug("Error while attempting to run nodejs: %s", e)
 
         # docker failed and nodejs not on system
         if nodejs is None:
             raise JavascriptException(
                 "NodeJSEngine requires Node.js engine to evaluate and validate "
                 "Javascript expressions, but couldn't find it.  Tried {trynodes}, "
-                f"{container_engine} run node:slim".format(
+                f"{container_engine} run node:alpine".format(
                     trynodes=", ".join(trynodes), container_engine=container_engine
                 )
             )
@@ -492,7 +495,7 @@ class NodeJSEngine(JSEngine):
             if not m:
                 return current_value
             next_segment_str = m.group(1)
-            key = None  # type: Optional[Union[str, int]]
+            key: Optional[Union[str, int]] = None
             if next_segment_str[0] == ".":
                 key = next_segment_str[1:]
             elif next_segment_str[1] in ("'", '"'):
@@ -511,7 +514,7 @@ class NodeJSEngine(JSEngine):
                     )
                 if key not in current_value:
                     raise WorkflowException(
-                        f"{parsed_string} does not contain key '{key}'"
+                        f"{parsed_string} does not contain key {key!r}."
                     )
             else:
                 try:
@@ -535,10 +538,10 @@ class NodeJSEngine(JSEngine):
                         remaining_string[m.end(1) :],
                         cast(CWLOutputType, current_value[cast(str, key)]),
                     )
-                except KeyError:
+                except KeyError as exc:
                     raise WorkflowException(
-                        f"{parsed_string} doesn't have property {key}"
-                    )
+                        f"{parsed_string!r} doesn't have property {key!r}."
+                    ) from exc
             elif isinstance(current_value, list) and isinstance(key, int):
                 try:
                     return self.regex_eval(
@@ -546,12 +549,14 @@ class NodeJSEngine(JSEngine):
                         remaining_string[m.end(1) :],
                         current_value[key],
                     )
-                except KeyError:
+                except KeyError as exc:
                     raise WorkflowException(
-                        f"{parsed_string} doesn't have property {key}"
-                    )
+                        f"{parsed_string!r} doesn't have property {key!r}."
+                    ) from exc
             else:
-                raise WorkflowException(f"{parsed_string} doesn't have property {key}")
+                raise WorkflowException(
+                    f"{parsed_string!r} doesn't have property {key!r}."
+                )
         else:
             return current_value
 
