@@ -3,7 +3,7 @@
 #  Copyright (c) 2020 Seven Bridges
 #  See https://github.com/rabix/sbpack/blob/b8404a0859ffcbe1edae6d8f934e51847b003320/LICENSE
 """
-Valid forms of user defined types stored in external file
+Valid forms of user defined types stored in external file.
 
 A single dictionary (tests/types/singletype.yml)
 A list of dictionaries (e.g. tests/types/recursive.yml)
@@ -145,7 +145,6 @@ def _inline_type(
     _inline_type_name_uniq_id += 1
 
     if isinstance(v, str):
-
         # Handle syntactic sugar
         if v.endswith("[]"):
             return {
@@ -171,7 +170,7 @@ def _inline_type(
 
         if path not in user_defined_types:
             raise RuntimeError(
-                f"Could not find type '{path}' in {str(user_defined_types)}"
+                f"Could not find type {path!r} in {user_defined_types!r}."
             )
         else:
             resolve_type = deepcopy(user_defined_types[path])
