@@ -1,6 +1,5 @@
 # SPDX-License-Identifier: Apache-2.0
 """Tests for cwl-docker-extract."""
-from pathlib import Path
 from shutil import which
 from tempfile import TemporaryDirectory
 
@@ -10,14 +9,15 @@ import cwl_utils.parser.cwl_v1_0 as parser
 from cwl_utils.docker_extract import traverse
 from cwl_utils.image_puller import DockerImagePuller, SingularityImagePuller
 
-HERE = Path(__file__).resolve().parent
-TEST_CWL = HERE / "../testdata/md5sum.cwl"
+from .util import get_data
+
+TEST_CWL = get_data("testdata/md5sum.cwl")
 
 
 @mark.skipif(which("docker") is None, reason="docker is not available")
 def test_traverse_workflow() -> None:
     """Test container extraction tool using Docker."""
-    loaded = parser.load_document(str(TEST_CWL.resolve()))
+    loaded = parser.load_document(TEST_CWL)
 
     with TemporaryDirectory() as tmpdir:
         reqs = set(traverse(loaded))
@@ -32,7 +32,7 @@ def test_traverse_workflow() -> None:
 @mark.skipif(which("singularity") is None, reason="singularity is not available")
 def test_traverse_workflow_singularity() -> None:
     """Test container extraction tool using Singularity."""
-    loaded = parser.load_document(str(TEST_CWL.resolve()))
+    loaded = parser.load_document(TEST_CWL)
 
     with TemporaryDirectory() as tmpdir:
         reqs = set(traverse(loaded))
