@@ -4,6 +4,7 @@ import tempfile
 from pathlib import Path
 from typing import MutableSequence, cast
 
+import pytest
 from pytest import raises
 from schema_salad.exceptions import ValidationException
 
@@ -13,10 +14,114 @@ import cwl_utils.parser.cwl_v1_1
 import cwl_utils.parser.cwl_v1_1_utils
 import cwl_utils.parser.cwl_v1_2
 import cwl_utils.parser.cwl_v1_2_utils
+import cwl_utils.parser.utils
 from cwl_utils.errors import WorkflowException
 from cwl_utils.parser import load_document_by_uri
 
-HERE = Path(__file__).resolve().parent
+from .util import get_data
+
+
+@pytest.mark.parametrize("cwlVersion", ["v1_0", "v1_1", "v1_2"])
+def test_static_checker_fail(cwlVersion: str) -> None:
+    """Confirm that static type checker raises expected exception."""
+    if cwlVersion == "v1_0":
+        with pytest.raises(
+            ValidationException,
+            match="\nSource .* of type .* is incompatible\n.*with sink .* of type .*",
+        ):
+            uri = Path(get_data("testdata/checker_wf/broken-wf.cwl")).resolve().as_uri()
+            cwl_obj = load_document_by_uri(uri)
+            cwl_utils.parser.utils.static_checker(cwl_obj)
+
+        with pytest.raises(
+            ValidationException, match="param .* not found in class: CommandLineTool"
+        ):
+            uri = (
+                Path(get_data("testdata/checker_wf/broken-wf2.cwl")).resolve().as_uri()
+            )
+            cwl_obj = load_document_by_uri(uri)
+            cwl_utils.parser.utils.static_checker(cwl_obj)
+
+        with pytest.raises(
+            ValidationException, match="param .* not found in class: Workflow"
+        ):
+            uri = (
+                Path(get_data("testdata/checker_wf/broken-wf3.cwl")).resolve().as_uri()
+            )
+            cwl_obj = load_document_by_uri(uri)
+            cwl_utils.parser.utils.static_checker(cwl_obj)
+
+    with pytest.raises(
+        ValidationException,
+        match="\nSource .* of type .* is incompatible\n.*with sink .* of type .*",
+    ):
+        uri = (
+            Path(get_data(f"testdata/count-lines6-wf_{cwlVersion}.cwl"))
+            .resolve()
+            .as_uri()
+        )
+        cwl_obj = load_document_by_uri(uri)
+        cwl_utils.parser.utils.static_checker(cwl_obj)
+
+    with pytest.raises(
+        ValidationException,
+        match="\nSource .* of type .* is incompatible\n.*with sink .* of type .*",
+    ):
+        uri = (
+            Path(get_data(f"testdata/count-lines6-single-source-wf_{cwlVersion}.cwl"))
+            .resolve()
+            .as_uri()
+        )
+        cwl_obj = load_document_by_uri(uri)
+        cwl_utils.parser.utils.static_checker(cwl_obj)
+
+    if cwlVersion == "v1_2":
+        with pytest.raises(ValidationException, match=".* pickValue is first_non_null"):
+            uri = (
+                Path(get_data("testdata/cond-single-source-wf-003.1.cwl"))
+                .resolve()
+                .as_uri()
+            )
+            cwl_obj = load_document_by_uri(uri)
+            cwl_utils.parser.utils.static_checker(cwl_obj)
+
+    with pytest.raises(ValidationException, match=".* pickValue is the_only_non_null"):
+        uri = (
+            Path(get_data("testdata/cond-single-source-wf-004.1.cwl"))
+            .resolve()
+            .as_uri()
+        )
+        cwl_obj = load_document_by_uri(uri)
+        cwl_utils.parser.utils.static_checker(cwl_obj)
+
+
+@pytest.mark.parametrize("cwlVersion", ["v1_0", "v1_1", "v1_2"])
+def test_static_checker_success(cwlVersion: str) -> None:
+    """Confirm that static type checker correctly validates workflows."""
+    test_files = [
+        f"testdata/record-output-wf_{cwlVersion}.cwl",
+        f"testdata/step_valuefrom5_wf_{cwlVersion}.cwl",
+        f"testdata/step_valuefrom5_wf_with_id_{cwlVersion}.cwl",
+        f"testdata/stdout-wf_{cwlVersion}.cwl",
+        f"testdata/scatter-wf1_{cwlVersion}.cwl",
+        f"testdata/scatter-wf2_{cwlVersion}.cwl",
+        f"testdata/scatter-wf3_{cwlVersion}.cwl",
+        f"testdata/count-lines7-wf_{cwlVersion}.cwl",
+        f"testdata/count-lines7-single-source-wf_{cwlVersion}.cwl",
+    ]
+    if cwlVersion == "v1_2":
+        test_files.extend(
+            [
+                "testdata/cond-wf-003.1.cwl",
+                "testdata/cond-wf-004.1.cwl",
+                "testdata/cond-wf-005.1.cwl",
+                "testdata/cond-single-source-wf-005.1.cwl",
+            ]
+        )
+    for test_file in test_files:
+        uri = Path(get_data(test_file)).resolve().as_uri()
+        cwl_obj = load_document_by_uri(uri)
+        cwl_utils.parser.utils.static_checker(cwl_obj)
 
 
 def test_v1_0_file_content_64_kB() -> None:
@@ -131,7 +236,7 @@ def test_v1_0_stderr_to_file_preserve_original() -> None:
 
 def test_v1_0_type_compare_list() -> None:
     """Test that the type comparison works correctly a list type with CWL v1.0."""
-    uri = Path(HERE / "../testdata/echo_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/echo_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
     assert cwl_utils.parser.cwl_v1_0_utils._compare_type(
         cwl_obj.inputs[0].type, cwl_obj.inputs[0].type
@@ -140,9 +245,9 @@ def test_v1_0_type_compare_list() -> None:
 
 def test_v1_0_type_compare_record() -> None:
     """Test that the type comparison works correctly a record type with CWL v1.0."""
-    uri = Path(HERE / "../testdata/record-output-wf_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/record-output-wf_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -151,9 +256,9 @@ def test_v1_0_type_compare_record() -> None:
 
 def test_v1_0_type_for_source() -> None:
     """Test that the type is correctly inferred from a source id with CWL v1.0."""
-    uri = Path(HERE / "../testdata/step_valuefrom5_wf_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/step_valuefrom5_wf_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.loadingOptions.fileuri + "#step1/echo_out_file"
     )
     assert source_type == "File"
@@ -162,12 +267,12 @@ def test_v1_0_type_for_source() -> None:
 def test_v1_0_type_for_source_with_id() -> None:
     """Test that the type is correctly inferred from a source id with CWL v1.0."""
     uri = (
-        Path(HERE / "../testdata/step_valuefrom5_wf_with_id_v1_0.cwl")
+        Path(get_data("testdata/step_valuefrom5_wf_with_id_v1_0.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.loadingOptions.fileuri + "#step1/echo_out_file"
     )
     assert source_type == "File"
@@ -175,9 +280,9 @@ def test_v1_0_type_for_source_with_id() -> None:
 
 def test_v1_0_type_for_stdout() -> None:
     """Test that the `stdout` type is correctly matched with the `File` type in CWL v1.0."""
-    uri = Path(HERE / "../testdata/stdout-wf_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/stdout-wf_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.outputs[0].outputSource
     )
     assert source_type == "File"
@@ -185,9 +290,9 @@ def test_v1_0_type_for_stdout() -> None:
 
 def test_v1_0_type_output_source_record() -> None:
     """Test that the type is correctly inferred from a record output source with CWL v1.0."""
-    uri = Path(HERE / "../testdata/record-output-wf_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/record-output-wf_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -202,9 +307,9 @@ def test_v1_0_type_output_source_record() -> None:
 
 def test_v1_0_type_for_output_source_with_single_scatter_step() -> None:
     """Test that the type is correctly inferred from a single scatter step with CWL v1.0."""
-    uri = Path(HERE / "../testdata/scatter-wf1_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf1_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -214,9 +319,9 @@ def test_v1_0_type_for_output_source_with_single_scatter_step() -> None:
 
 def test_v1_0_type_for_output_source_with_nested_crossproduct_scatter_step() -> None:
     """Test that the type is correctly inferred from a nested_crossproduct scatter step with CWL v1.0."""
-    uri = Path(HERE / "../testdata/scatter-wf2_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf2_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -227,9 +332,9 @@ def test_v1_0_type_for_output_source_with_nested_crossproduct_scatter_step() ->
 
 def test_v1_0_type_for_output_source_with_flat_crossproduct_scatter_step() -> None:
     """Test that the type is correctly inferred from a flat_crossproduct scatter step with CWL v1.0."""
-    uri = Path(HERE / "../testdata/scatter-wf3_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf3_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj, sourcenames=cwl_obj.outputs[0].outputSource
     )
     assert isinstance(source_type, cwl_utils.parser.cwl_v1_0.ArraySchema)
@@ -238,9 +343,9 @@ def test_v1_0_type_for_output_source_with_flat_crossproduct_scatter_step() -> No
 
 def test_v1_0_type_for_source_with_multiple_entries_merge_nested() -> None:
     """Test that the type is correctly inferred from a list of source ids and merge_nested with CWL v1.0."""
-    uri = Path(HERE / "../testdata/count-lines6-wf_v1_0.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/count-lines6-wf_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -251,10 +356,10 @@ def test_v1_0_type_for_source_with_multiple_entries_merge_nested() -> None:
 
 
 def test_v1_0_type_for_source_with_multiple_entries_merge_flattened() -> None:
-    """Test that the type is correctly inferred from a list of source ids and merge_flattened with CWL v1.0."""
-    uri = Path(HERE / "../testdata/count-lines7-wf_v1_0.cwl").resolve().as_uri()
+    """Test that the type is correctly inferred from a list of source ids and merge_flattened with CWL v1.0."""  # noqa: B950
+    uri = Path(get_data("testdata/count-lines7-wf_v1_0.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -266,12 +371,12 @@ def test_v1_0_type_for_source_with_multiple_entries_merge_flattened() -> None:
 def test_v1_0_type_for_source_with_single_entry_merge_nested() -> None:
     """Test that the type is correctly inferred from a single source id and merge_nested with CWL v1.0."""
     uri = (
-        Path(HERE / "../testdata/count-lines6-single-source-wf_v1_0.cwl")
+        Path(get_data("testdata/count-lines6-single-source-wf_v1_0.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -284,12 +389,12 @@ def test_v1_0_type_for_source_with_single_entry_merge_nested() -> None:
 def test_v1_0_type_for_source_with_single_entry_merge_flattened() -> None:
     """Test that the type is correctly inferred from a single source id and merge_flattened with CWL v1.0."""
     uri = (
-        Path(HERE / "../testdata/count-lines7-single-source-wf_v1_0.cwl")
+        Path(get_data("testdata/count-lines7-single-source-wf_v1_0.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_0_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -453,7 +558,7 @@ def test_v1_1_stdin_to_file_fail_with_original() -> None:
 
 def test_v1_1_type_compare_list() -> None:
     """Test that the type comparison works correctly a list type with CWL v1.1."""
-    uri = Path(HERE / "../testdata/echo_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/echo_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
     assert cwl_utils.parser.cwl_v1_1_utils._compare_type(
         cwl_obj.inputs[0].type, cwl_obj.inputs[0].type
@@ -462,9 +567,9 @@ def test_v1_1_type_compare_list() -> None:
 
 def test_v1_1_type_compare_record() -> None:
     """Test that the type comparison works correctly a record type with CWL v1.1."""
-    uri = Path(HERE / "../testdata/record-output-wf_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/record-output-wf_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -473,9 +578,9 @@ def test_v1_1_type_compare_record() -> None:
 
 def test_v1_1_type_for_source() -> None:
     """Test that the type is correctly inferred from a source id with CWL v1.1."""
-    uri = Path(HERE / "../testdata/step_valuefrom5_wf_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/step_valuefrom5_wf_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.loadingOptions.fileuri + "#step1/echo_out_file"
     )
     assert source_type == "File"
@@ -484,12 +589,12 @@ def test_v1_1_type_for_source() -> None:
 def test_v1_1_type_for_source_with_id() -> None:
     """Test that the type is correctly inferred from a source id with CWL v1.1."""
     uri = (
-        Path(HERE / "../testdata/step_valuefrom5_wf_with_id_v1_1.cwl")
+        Path(get_data("testdata/step_valuefrom5_wf_with_id_v1_1.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.loadingOptions.fileuri + "#step1/echo_out_file"
     )
     assert source_type == "File"
@@ -497,9 +602,9 @@ def test_v1_1_type_for_source_with_id() -> None:
 
 def test_v1_1_type_for_stdout() -> None:
     """Test that the `stdout` type is correctly matched with the `File` type in CWL v1.1."""
-    uri = Path(HERE / "../testdata/stdout-wf_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/stdout-wf_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.outputs[0].outputSource
     )
     assert source_type == "File"
@@ -507,9 +612,9 @@ def test_v1_1_type_for_stdout() -> None:
 
 def test_v1_1_type_output_source_record() -> None:
     """Test that the type is correctly inferred from a record output source with CWL v1.1."""
-    uri = Path(HERE / "../testdata/record-output-wf_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/record-output-wf_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -524,9 +629,9 @@ def test_v1_1_type_output_source_record() -> None:
 
 def test_v1_1_type_for_output_source_with_single_scatter_step() -> None:
     """Test that the type is correctly inferred from a single scatter step with CWL v1.1."""
-    uri = Path(HERE / "../testdata/scatter-wf1_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf1_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -536,9 +641,9 @@ def test_v1_1_type_for_output_source_with_single_scatter_step() -> None:
 
 def test_v1_1_type_for_output_source_with_nested_crossproduct_scatter_step() -> None:
     """Test that the type is correctly inferred from a nested_crossproduct scatter step with CWL v1.1."""
-    uri = Path(HERE / "../testdata/scatter-wf2_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf2_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -549,9 +654,9 @@ def test_v1_1_type_for_output_source_with_nested_crossproduct_scatter_step() ->
 
 def test_v1_1_type_for_output_source_with_flat_crossproduct_scatter_step() -> None:
     """Test that the type is correctly inferred from a flat_crossproduct scatter step with CWL v1.1."""
-    uri = Path(HERE / "../testdata/scatter-wf3_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf3_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -561,9 +666,9 @@ def test_v1_1_type_for_output_source_with_flat_crossproduct_scatter_step() -> No
 
 def test_v1_1_type_for_source_with_multiple_entries_merge_nested() -> None:
     """Test that the type is correctly inferred from a list of source ids and merge_nested with CWL v1.1."""
-    uri = Path(HERE / "../testdata/count-lines6-wf_v1_1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/count-lines6-wf_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -574,10 +679,10 @@ def test_v1_1_type_for_source_with_multiple_entries_merge_nested() -> None:
 
 
 def test_v1_1_type_for_source_with_multiple_entries_merge_flattened() -> None:
-    """Test that the type is correctly inferred from a list of source ids and merge_flattened with CWL v1.1."""
-    uri = Path(HERE / "../testdata/count-lines7-wf_v1_1.cwl").resolve().as_uri()
+    """Test that the type is correctly inferred from a list of source ids and merge_flattened with CWL v1.1."""  # noqa: B950
+    uri = Path(get_data("testdata/count-lines7-wf_v1_1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -589,12 +694,12 @@ def test_v1_1_type_for_source_with_multiple_entries_merge_flattened() -> None:
 def test_v1_1_type_for_source_with_single_entry_merge_nested() -> None:
     """Test that the type is correctly inferred from a single source id and merge_nested with CWL v1.1."""
     uri = (
-        Path(HERE / "../testdata/count-lines6-single-source-wf_v1_1.cwl")
+        Path(get_data("testdata/count-lines6-single-source-wf_v1_1.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -607,12 +712,12 @@ def test_v1_1_type_for_source_with_single_entry_merge_nested() -> None:
 def test_v1_1_type_for_source_with_single_entry_merge_flattened() -> None:
     """Test that the type is correctly inferred from a single source id and merge_flattened with CWL v1.1."""
     uri = (
-        Path(HERE / "../testdata/count-lines7-single-source-wf_v1_1.cwl")
+        Path(get_data("testdata/count-lines7-single-source-wf_v1_1.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_1_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -776,7 +881,7 @@ def test_v1_2_stdin_to_file_fail_with_original() -> None:
 
 def test_v1_2_type_compare_list() -> None:
     """Test that the type comparison works correctly a list type with CWL v1.2."""
-    uri = Path(HERE / "../testdata/echo_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/echo_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
     assert cwl_utils.parser.cwl_v1_2_utils._compare_type(
         cwl_obj.inputs[0].type, cwl_obj.inputs[0].type
@@ -785,9 +890,9 @@ def test_v1_2_type_compare_list() -> None:
 
 def test_v1_2_type_compare_record() -> None:
     """Test that the type comparison works correctly a record type with CWL v1.2."""
-    uri = Path(HERE / "../testdata/record-output-wf_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/record-output-wf_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -796,9 +901,9 @@ def test_v1_2_type_compare_record() -> None:
 
 def test_v1_2_type_for_source() -> None:
     """Test that the type is correctly inferred from a source id with CWL v1.2."""
-    uri = Path(HERE / "../testdata/step_valuefrom5_wf_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/step_valuefrom5_wf_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.loadingOptions.fileuri + "#step1/echo_out_file"
     )
     assert source_type == "File"
@@ -807,12 +912,12 @@ def test_v1_2_type_for_source() -> None:
 def test_v1_2_type_for_source_with_id() -> None:
     """Test that the type is correctly inferred from a source id with CWL v1.2."""
     uri = (
-        Path(HERE / "../testdata/step_valuefrom5_wf_with_id_v1_2.cwl")
+        Path(get_data("testdata/step_valuefrom5_wf_with_id_v1_2.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.loadingOptions.fileuri + "#step1/echo_out_file"
     )
     assert source_type == "File"
@@ -820,9 +925,9 @@ def test_v1_2_type_for_source_with_id() -> None:
 
 def test_v1_2_type_for_stdout() -> None:
     """Test that the `stdout` type is correctly matched with the `File` type in CWL v1.2."""
-    uri = Path(HERE / "../testdata/stdout-wf_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/stdout-wf_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         cwl_obj, cwl_obj.outputs[0].outputSource
     )
     assert source_type == "File"
@@ -830,9 +935,9 @@ def test_v1_2_type_for_stdout() -> None:
 
 def test_v1_2_type_output_source_record() -> None:
     """Test that the type is correctly inferred from a record output source with CWL v1.2."""
-    uri = Path(HERE / "../testdata/record-output-wf_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/record-output-wf_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -847,9 +952,9 @@ def test_v1_2_type_output_source_record() -> None:
 
 def test_v1_2_type_for_output_source_with_single_scatter_step() -> None:
     """Test that the type is correctly inferred from a single scatter step with CWL v1.2."""
-    uri = Path(HERE / "../testdata/scatter-wf1_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf1_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -859,9 +964,9 @@ def test_v1_2_type_for_output_source_with_single_scatter_step() -> None:
 
 def test_v1_2_type_for_output_source_with_nested_crossproduct_scatter_step() -> None:
     """Test that the type is correctly inferred from a nested_crossproduct scatter step with CWL v1.2."""
-    uri = Path(HERE / "../testdata/scatter-wf2_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf2_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -872,9 +977,9 @@ def test_v1_2_type_for_output_source_with_nested_crossproduct_scatter_step() ->
 
 def test_v1_2_type_for_output_source_with_flat_crossproduct_scatter_step() -> None:
     """Test that the type is correctly inferred from a flat_crossproduct scatter step with CWL v1.2."""
-    uri = Path(HERE / "../testdata/scatter-wf3_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/scatter-wf3_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
     )
@@ -884,9 +989,9 @@ def test_v1_2_type_for_output_source_with_flat_crossproduct_scatter_step() -> No
 
 def test_v1_2_type_for_source_with_multiple_entries_merge_nested() -> None:
     """Test that the type is correctly inferred from a list of source ids and merge_nested with CWL v1.2."""
-    uri = Path(HERE / "../testdata/count-lines6-wf_v1_2.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/count-lines6-wf_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -897,10 +1002,10 @@ def test_v1_2_type_for_source_with_multiple_entries_merge_nested() -> None:
 
 
 def test_v1_2_type_for_source_with_multiple_entries_merge_flattened() -> None:
-    """Test that the type is correctly inferred from a list of source ids and merge_flattened with CWL v1.2."""
-    uri = Path(HERE / "../testdata/count-lines7-wf_v1_2.cwl").resolve().as_uri()
+    """Test that the type is correctly inferred from a list of source ids and merge_flattened with CWL v1.2."""  # noqa: B950
+    uri = Path(get_data("testdata/count-lines7-wf_v1_2.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -912,12 +1017,12 @@ def test_v1_2_type_for_source_with_multiple_entries_merge_flattened() -> None:
 def test_v1_2_type_for_source_with_single_entry_merge_nested() -> None:
     """Test that the type is correctly inferred from a single source id and merge_nested with CWL v1.2."""
     uri = (
-        Path(HERE / "../testdata/count-lines6-single-source-wf_v1_2.cwl")
+        Path(get_data("testdata/count-lines6-single-source-wf_v1_2.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -930,12 +1035,12 @@ def test_v1_2_type_for_source_with_single_entry_merge_nested() -> None:
 def test_v1_2_type_for_source_with_single_entry_merge_flattened() -> None:
     """Test that the type is correctly inferred from a single source id and merge_flattened with CWL v1.2."""
     uri = (
-        Path(HERE / "../testdata/count-lines7-single-source-wf_v1_2.cwl")
+        Path(get_data("testdata/count-lines7-single-source-wf_v1_2.cwl"))
         .resolve()
         .as_uri()
     )
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.steps[0].in_[0].source,
         linkMerge=cwl_obj.steps[0].in_[0].linkMerge,
@@ -946,9 +1051,9 @@ def test_v1_2_type_for_source_with_single_entry_merge_flattened() -> None:
 
 def test_v1_2_type_for_source_with_multiple_entries_first_non_null() -> None:
     """Test that the type is correctly inferred from a list of source ids and first_non_null with CWL v1.2."""
-    uri = Path(HERE / "../testdata/cond-wf-003.1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/cond-wf-003.1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
         pickValue=cwl_obj.outputs[0].pickValue,
@@ -957,10 +1062,10 @@ def test_v1_2_type_for_source_with_multiple_entries_first_non_null() -> None:
 
 
 def test_v1_2_type_for_source_with_multiple_entries_the_only_non_null() -> None:
-    """Test that the type is correctly inferred from a list of source ids and the_only_non_null with CWL v1.2."""
-    uri = Path(HERE / "../testdata/cond-wf-004.1.cwl").resolve().as_uri()
+    """Test that the type is correctly inferred from a list of source ids and the_only_non_null with CWL v1.2."""  # noqa: B950
+    uri = Path(get_data("testdata/cond-wf-004.1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
         pickValue=cwl_obj.outputs[0].pickValue,
@@ -970,9 +1075,9 @@ def test_v1_2_type_for_source_with_multiple_entries_the_only_non_null() -> None:
 
 def test_v1_2_type_for_source_with_multiple_entries_all_non_null() -> None:
     """Test that the type is correctly inferred from a list of source ids and all_non_null with CWL v1.2."""
-    uri = Path(HERE / "../testdata/cond-wf-005.1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/cond-wf-005.1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
         pickValue=cwl_obj.outputs[0].pickValue,
@@ -983,9 +1088,9 @@ def test_v1_2_type_for_source_with_multiple_entries_all_non_null() -> None:
 
 def test_v1_2_type_for_source_with_single_entry_first_non_null() -> None:
     """Test that the type is correctly inferred from a single source id and first_non_null with CWL v1.2."""
-    uri = Path(HERE / "../testdata/cond-single-source-wf-003.1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/cond-single-source-wf-003.1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
         pickValue=cwl_obj.outputs[0].pickValue,
@@ -994,10 +1099,10 @@ def test_v1_2_type_for_source_with_single_entry_first_non_null() -> None:
 
 
 def test_v1_2_type_for_source_with_single_entry_the_only_non_null() -> None:
-    """Test that the type is correctly inferred from a single source id and the_only_non_null with CWL v1.2."""
-    uri = Path(HERE / "../testdata/cond-single-source-wf-004.1.cwl").resolve().as_uri()
+    """Test that the type is correctly inferred from a single source id and the_only_non_null with CWL v1.2."""  # noqa: B950
+    uri = Path(get_data("testdata/cond-single-source-wf-004.1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
         pickValue=cwl_obj.outputs[0].pickValue,
@@ -1007,9 +1112,9 @@ def test_v1_2_type_for_source_with_single_entry_the_only_non_null() -> None:
 
 def test_v1_2_type_for_source_with_single_entry_all_non_null() -> None:
     """Test that the type is correctly inferred from a single source id and all_non_null with CWL v1.2."""
-    uri = Path(HERE / "../testdata/cond-single-source-wf-005.1.cwl").resolve().as_uri()
+    uri = Path(get_data("testdata/cond-single-source-wf-005.1.cwl")).resolve().as_uri()
     cwl_obj = load_document_by_uri(uri)
-    source_type = cwl_utils.parser.cwl_v1_2_utils.type_for_source(
+    source_type = cwl_utils.parser.utils.type_for_source(
         process=cwl_obj,
         sourcenames=cwl_obj.outputs[0].outputSource,
         pickValue=cwl_obj.outputs[0].pickValue,
